#!/bin/bash
# Executes a database backup for the specified database container.
# The current directory is expected to contain a project configured
# as with SSDT conventions for an application database.
#
# When successful, the output file will be in ./backup with the
# container name and timestamp in the file.
# The file will also be placed on the specified remote target
# The format of the output is a compressed pg_dump (sql) format.
#
#
# This will backup both usasdb and uspsdb,
# along with the top level contents of the project directory
#Environment variables can be used for REMOTE_BACKUP_TARGET and REMOTE_USERNAME
remoteTarget=${1:-$REMOTE_BACKUP_TARGET}
userName=${2:-$REMOTE_USERNAME}
projectDir=${3:-$PWD}


cd $projectDir

source "${SSDT_SCRIPTS:-$(dirname "${BASH_SOURCE[0]}")}/.functions.sh"

set -o pipefail
mkdir -p ./backup

container1=usasdb
container2=uspsdb

project=$(composeGetProject)

#If the project is empty, we want to stop the process because this is being run from the wrong directory

if [ "$project" == "" ]; then
   echo "no project available"
   exit 1
fi

## first will try to get IRNfrom usasdb; if that is empty it will try to get it from uspsdb

IRN=$(docker-compose exec -T $container1 psql --username=postgres --dbname=$container1 -t -c 'select irn from organization') 

if [ "$IRN" == "" ]; then
IRN=$(docker-compose exec -T $container2 psql --username=postgres --dbname=$container2 -t -c 'select irn from organization') 
fi

##Trim function in postgres didn't work - so take out the extra space this way.
IRN=`echo $IRN | sed -e 's/^[[:space:]]*//'`

## if both database are empty, it will put IRN 000000
if [ "$IRN" == "" ]; then
   echo "no IRN set, using 000000"
    IRN=000000
fi

echo "project is $project"
echo "Containers are $container1 and $container2"
echo "Remote target is $remoteTarget"
echo "Username is $userName"
echo "IRN is $IRN"


backupFile1=./backup/${IRN}${project}-${container1}.$(date +%Y-%m-%d-%H-%M-%S).backup
backupFile2=./backup/${IRN}${project}-${container2}.$(date +%Y-%m-%d-%H-%M-%S).backup
backupFile3=./backup/${IRN}${project}-directory.$(date +%Y-%m-%d-%H-%M-%S).directorycontents.tar.gz

#usasdb
echo "starting backup of $container1 for $project"
docker-compose exec -T $container1 sh -c "gosu postgres pg_dump -Cc --if-exists --dbname=$container1 ; (exit $?) " > ${backupFile1}

  if [[ $( grep --count "CREATE TABLE" ${backupFile1} ) -lt 200 || $( grep --count "PostgreSQL database dump complete" ${backupFile1} ) -eq 0 ]]; then
     echo "ERROR: backup verification FAILED"
     echo "ERROR:  $(tail ${backupFile1})"
	 echo "Error: empty database"
	 #else  We still gzip the backup file
  fi
     gzip ${backupFile1}

     echo "completed backup of $container1 for $project to ${backupFile1}"



#####
#uspsdb

echo "starting backup of $container2 for $project"
docker-compose exec -T $container2 sh -c "gosu postgres pg_dump -Cc --if-exists --dbname=$container2 ; (exit $?) " > ${backupFile2}

  if [[ $( grep --count "CREATE TABLE" ${backupFile2} ) -lt 200 || $( grep --count "PostgreSQL database dump complete" ${backupFile2} ) -eq 0 ]]; then
     echo "ERROR: backup verification FAILED"
     echo "ERROR:  $(tail ${backupFile2})"
	 echo "Error: empty database"
     #else  We still gzip the backup file
  fi	 
     gzip ${backupFile2}

     echo "completed backup of $container2 for $project to ${backupFile2}"

  

#######

#backup of all files in current directory
echo "starting backup of current directory for $project"

#tar -czf ${backupFile3} . --exclude=./backup
tar --exclude=./backup -czf ${backupFile3} . 

echo "completed backup of all files for $project to ${backupFile3}"

#Create remote directory before scp
host=`echo $userName@$remoteTarget | sed -e 's/:.*//'`

target=`echo $remoteTarget | sed -e 's/.*://'`


ssh $host mkdir -p $target/$project

#SCP files
#
scp ${backupFile1}.gz  ${backupFile2}.gz ${backupFile3} $userName@$remoteTarget/$project

echo " "

echo "completed sending ${backupFile1}.gz, ${backupFile2}.gz, and ${backupFile3} to ${remoteTarget}/${project} as user $userName"


